#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars - This software is public domain

import os
import sys
import string
import json
import csv
import time
import six
from collections import OrderedDict
from datetime import datetime


def new_pjson_deactive_stub():
    ps = OrderedDict()

    # ProviderJSON stub
    ps['enumeration_type'] = ""
    ps['number'] = ""
    ps['title'] = "Deactive"
    ps["basic"] = OrderedDict()
    ps["basic"]["status"] = "D"
    ps["basic"]["deactivation_date"] = ""
    return ps


def new_pjson_stub():

    ps = OrderedDict()

    # ProviderJSON stub
    ps['enumeration_type'] = ""
    ps['number'] = ""
    ps['title'] = ""
    ps['last_updated_epoch'] = ""
    ps['created_epoch'] = ""
    ps["basic"] = OrderedDict()
    ps["basic"]["name_prefix"] = ""
    ps["basic"]["first_name"] = ""
    ps["basic"]["last_name"] = ""
    ps["basic"]["middle_name"] = ""
    ps["basic"]["name_suffix"] = ""
    ps["basic"]["credential"] = ""
    ps["basic"]["sole_proprietor"] = ""
    ps["basic"]["organization_name"] = ""
    ps["basic"]["organization_other_name"] = ""
    ps["basic"]["organization_other_name_code"] = ""
    ps["basic"]["organizational_subpart"] = ""
    ps["basic"]["ssn"] = ""
    ps["basic"]["ein"] = ""
    ps["basic"]["itin"] = ""
    ps["basic"]["gender"] = ""
    ps["basic"]["date_of_birth"] = ""
    ps["basic"]["state_of_birth"] = ""
    ps["basic"]["country_of_birth"] = ""
    ps["basic"]["number"] = ""
    ps["basic"]["initial_enumeration_date"] = ""
    ps["basic"]["enumeration_date"] = ""
    ps["basic"]["last_updated"] = ""
    ps["basic"]["date_of_death"] = ""
    ps["basic"]["reactivation_date"] = ""
    ps["basic"]["status"] = "A"
    ps["basic"]["deactivated_details"] = ""
    ps["basic"]["deactivation_date"] = "None"
    ps["basic"]["deactivation_reason_code"] = ""
    ps["basic"]["decativation_note"] = ""
    ps["basic"]["deceased_notes"] = ""
    ps["basic"]["parent_organization"] = ""
    ps["basic"]["parent_organization_ein"] = ""
    ps["basic"]["parent_organization_legal_business_name"] = ""
    ps["basic"]["recativation_note"] = ""
    ps["basic"]["comments"] = ""
    ps["basic"]["authorized_official_credential"] = ""
    ps["basic"]["authorized_official_email"] = ""
    ps["basic"]["authorized_official_first_name"] = ""
    ps["basic"]["authorized_official_last_name"] = ""
    ps["basic"]["authorized_official_middle_name"] = ""
    ps["basic"]["authorized_official_prefix"] = ""
    ps["basic"]["authorized_official_suffix"] = ""
    ps["basic"]["authorized_official_telephone_number"] = ""
    ps["basic"]["authorized_official_telephone_extension"] = ""
    ps["basic"]["authorized_official_title"] = ""
    ps["basic"]["authorized_official_title_or_position"] = ""
    ps["basic"]["contact_person_credential"] = ""
    ps["basic"]["contact_person_email"] = ""
    ps["basic"]["contact_person_first_name"] = ""
    ps["basic"]["contact_person_last_name"] = ""
    ps["basic"]["contact_person_middle_name"] = ""
    ps["basic"]["contact_person_prefix"] = ""
    ps["basic"]["contact_person_suffix"] = ""
    ps["basic"]["contact_person_telephone_extension"] = ""
    ps["basic"]["contact_person_telephone_number"] = ""
    ps["basic"]["contact_person_title"] = ""
    ps["basic"]["contact_person_title_or_position"] = ""
    ps["other_names"] = []
    ps["addresses"] = []
    ps["taxonomies"] = []
    ps["licenses"] = []
    ps["taxonomy_licenses"] = []
    ps["identifiers"] = []
    ps["taxonomy_groups"] = []
    return ps


def publiccsv2pjson(csvfile, output_dir):
    """Return a response_dict with summary of  publiccsv2pjson transaction."""

    process_start_time = time.time()

    pdir = 1

    # make the output dir
    try:
        os.mkdir(output_dir)
    except:
        pass

    response_dict = OrderedDict()
    fh = open(csvfile, 'r')
    csvhandle = csv.reader(fh, delimiter=',')
    rowindex = 0
    po_count = 0
    error_list = []

    for row in csvhandle:
        if rowindex == 0:

            rowindex += 1
            column_headers = row

            cleaned_headers = []
            for c in column_headers:
                c = c.replace(".", "")
                c = c.replace("(", "")
                c = c.replace(")", "")
                c = c.replace("$", "-")
                c = c.replace(" ", "_")
                cleaned_headers.append(c)
        else:

            # If the records is not redacted (because its inactive)

            zip_record = list(zip(cleaned_headers, row))
            record = dict(list(zip(cleaned_headers, row)))

            # get rid of blanks
            no_blank_zip = []
            for i in zip_record:
                if i[1]:
                    no_blank_zip.append(i)

            # start our object off with a stub.

            if row[1]:
                p = new_pjson_stub()

                if row[1] == "1":
                    p["enumeration_type"] = "NPI-1"
                    p["title"] = "%s %s" % (row[6], row[5])
                    if row[10]:
                        p["title"] = "%s, %s" % (p["title"], row[10])

                elif row[1] == "2":
                    p["enumeration_type"] = "NPI-2"
                    p["title"] = row[4]

                else:
                    # The record is deactivated
                    p['basic']['status'] = "D"
                    p["title"] = "Deactivated"

                p["number"] = row[0]
                # Load basic
                for i in range(0, len(row)):
                    if row[i]:
                        row[i] = six.u(row[i])
                    else:
                        row[i] = ""

                # Provider name
                p["basic"]["name_prefix"] = row[8].capitalize()
                p["basic"]['first_name'] = row[6].upper()
                p["basic"]["middle_name"] = row[7].upper()
                p["basic"]['last_name'] = row[5].upper()
                p["basic"]["name_suffix"] = row[9]
                p["basic"]["credential"] = row[10]
                p["basic"]["gender"] = row[41].upper()

                if row[307] in (True, "Y", "y", "YES"):
                    p["basic"]["sole_proprietor"] = "YES"
                else:
                    p["basic"]["sole_proprietor"] = "NO"

                # provider_organization
                p["basic"]["organization_name"] = row[4]

                p["basic"]["ein"] = row[3]
                if p["basic"]["ein"] == "<UNAVAIL>":
                    p["basic"]["ein"] = ""

                p["basic"]["parent_organization_ein"] = row[310]
                p["basic"]["parent_organization_legal_business_name"] = row[309]

                p["basic"]["enumeration_date"] = row[36]
                if p["basic"]["enumeration_date"]:
                    p['created_epoch'] = int(datetime(int(p["basic"]["enumeration_date"][6:10]),
                                                      int(p["basic"][
                                                          "enumeration_date"][0:2]),
                                                      int(p["basic"][
                                                          "enumeration_date"][3:5]),
                                                      0, 0).strftime('%s'))

                p["basic"]["last_updated"] = row[37]
                if p["basic"]["last_updated"]:
                    p['last_updated_epoch'] = int(datetime(int(p["basic"]["last_updated"][6:10]),
                                                           int(p["basic"][
                                                               "last_updated"][0:2]),
                                                           int(p["basic"][
                                                               "last_updated"][3:5]),
                                                           0, 0).strftime('%s'))

                p["basic"]["deactivation_date"] = row[39]
                p["basic"]["reactivation_date"] = row[40]
                p["basic"]["deactivation_reason_code"] = row[38]

                if row[37]:
                    month = row[37][0:2]
                    day = row[37][3:5]
                    year = row[37][6:10]
                    p["basic"][
                        "last_updated"] = "%s-%s-%s" % (year, month, day)

                if row[36]:
                    month = row[36][0:2]
                    day = row[36][3:5]
                    year = row[36][6:10]
                    p["basic"][
                        "enumeration_date"] = "%s-%s-%s" % (year, month, day)

                if row[39]:
                    month = row[39][0:2]
                    day = row[39][3:5]
                    year = row[39][6:10]
                    p["basic"][
                        "deactivation_date"] = "%s-%s-%s" % (year, month, day)

                if row[40]:
                    month = row[40][0:2]
                    day = row[40][3:5]
                    year = row[40][6:10]
                    p["basic"][
                        "reactivation_date"] = "%s-%s-%s" % (year, month, day)

                # Contact person not included in public file

                # Authorized official
                p["basic"]["authorized_official_last_name"] = row[42].upper()
                p["basic"]["authorized_official_first_name"] = row[43].upper()
                p["basic"]["authorized_official_middle_name"] = row[44].upper()

                ao_phone_number = row[46].upper()

                ao_phone_number = "%s-%s-%s" % (ao_phone_number[0:3], ao_phone_number[
                                                3:6], ao_phone_number[6:12])

                if ao_phone_number != "--":
                    p["basic"]["authorized_official_telephone_number"] = ao_phone_number

                p["basic"]["authorized_official_title_or_position"] = row[45].upper()

                clean_basic = OrderedDict()
                for k, v in list(p["basic"].items()):
                    if v:
                        clean_basic[k] = v

                p["basic"] = clean_basic

                if row[11] or row[13] or row[14]:
                    other_name = OrderedDict()
                    if row[11]:
                        other_name["organization_name"] = row[11].upper()

                    if row[12]:
                        other_name["code"] = row[12].upper()
                    if row[19]:
                        other_name["code"] = row[19].upper()

                    if row[13]:
                        other_name["last_name"] = row[13].upper()
                    if row[14]:
                        other_name["first_name"] = row[14].upper()
                    if row[15]:
                        other_name["middle_name"] = row[15].upper()
                    if row[16]:
                        other_name["prefix"] = row[16].capitalize()
                    if row[17]:
                        other_name["suffix"] = row[17].capitalize()
                    if row[18]:
                        other_name["credential"] = row[18]
                    p["other_names"].append(other_name)

                # Addresses
                # location ---
                a = OrderedDict()
                a["country_code"] = row[33].upper()
                a["address_purpose"] = "LOCATION"

                if a["country_code"] == "US":
                    a["address_type"] = "DOM"
                    a["address_1"] = row[28].upper()
                    a["address_2"] = row[29].upper()
                    a["city"] = row[30].upper()
                    a["state"] = row[31].upper()
                    a["zip"] = row[32].upper()

                    if row[34]:

                        a["us_telephone_number"] = "%s-%s-%s" % (
                            row[34][0:3], row[34][3:6], row[34][6:12])

                    if row[35]:
                        a["us_fax_number"] = "%s-%s-%s" % (
                            row[35][0:3], row[35][3:6], row[35][6:12])

                else:
                    a["address_type"] = "FGN"
                    a["address_1"] = row[28].upper()
                    a["address_2"] = row[29].upper()
                    a["city"] = row[30].upper()
                    a["foreign_state"] = row[31].upper()
                    a["foreign_postal"] = row[32].upper()
                    a["foreign_telephone_number"] = row[34].upper()
                    a["foreign_fax_number"] = row[35].upper()

                p['addresses'].append(a)

                # Maiing address ---
                a = OrderedDict()
                a["country_code"] = row[25].upper()
                a["address_purpose"] = "MAILING"

                if a["country_code"] == "US":
                    a["address_type"] = "DOM"
                    a["address_1"] = row[20].upper()
                    a["address_2"] = row[21].upper()
                    a["city"] = row[22].upper()
                    a["state"] = row[23].upper()
                    a["zip"] = row[24].upper()

                    if row[26]:
                        a["us_telephone_number"] = "%s-%s-%s" % (
                            row[26][0:3], row[26][3:6], row[26][6:12])

                    if row[27]:

                        a["us_fax_number"] = "%s-%s-%s" % (
                            row[27][0:3], row[27][3:6], row[27][6:12])

                else:
                    a["address_type"] = "FGN"
                    a["address_1"] = row[20].upper()
                    a["address_2"] = row[21].upper()
                    a["city"] = row[22].upper()
                    a["foreign_state"] = row[23].upper()
                    a["foreign_postal"] = row[24].upper()
                    a["foreign_telephone_number"] = row[26].upper()
                    a["foreign_fax_number"] = row[27].upper()

                p['addresses'].append(a)

                # licenses and taxonomies--------------------------------------
                # starting at row 47, 2nd set starts at 51

                # Taxonomy
                taxonomy_code_position = 47
                primary_taxonomy_position = 50
                license_state_position = 49
                license_number_position = 48

                for i in range(1, 15):

                    if row[taxonomy_code_position]:
                        taxonomy = OrderedDict()
                        taxonomy['code'] = row[taxonomy_code_position]
                        if row[primary_taxonomy_position] in ("Y", "True", True, "YES"):
                            taxonomy['primary'] = True
                        else:
                            taxonomy['primary'] = False

                        p['taxonomies'].append(taxonomy)

                     # License
                    if row[license_number_position]:

                        license = OrderedDict()
                        mlvs = "%s-UNK-%s" % (row[license_state_position],
                                              row[license_number_position])
                        license['code'] = mlvs
                        license['state'] = row[license_state_position].upper()
                        license['type'] = "UNK"
                        license['status'] = "UNK"

                        p['licenses'].append(license)

                    #Taxonomy and License
                    if row[license_number_position] and row[taxonomy_code_position]:
                        taxonomy_license = OrderedDict()
                        taxonomy_license['taxonomy_code'] = row[
                            taxonomy_code_position]
                        mlvs = "%s-UNK-%s" % (row[license_state_position],
                                              row[license_number_position])
                        taxonomy_license['license_code'] = mlvs
                        p['taxonomy_licenses'].append(taxonomy_license)

                    taxonomy_code_position += 4
                    primary_taxonomy_position += 4
                    license_state_position += 4
                    license_number_position += 4

                # identifiers----------------
                # starting at row 107

                identifier_position = 107
                identifier_code_position = 108
                identifier_state_position = 109
                identifier_issuer_position = 110

                for i in range(1, 50):

                    if row[identifier_position]:
                        identifier = OrderedDict()
                        identifier['identifier'] = row[identifier_position]
                        identifier['code'] = row[identifier_code_position]
                        identifier['state'] = row[
                            identifier_state_position].upper()
                        identifier['issuer'] = row[identifier_issuer_position]
                        p['identifiers'].append(identifier)

                    identifier_position += 4
                    identifier_code_position += 4
                    identifier_state_position += 4
                    identifier_issuer_position += 4

                # Get Taxonomy Groups for Type 2
                for i in range(314, 328):
                    if row[i]:
                        code, description = row[i].split(" ", 1)
                        tg = {"code": code, "description": description}
                        p['taxonomy_groups'].append(tg)

                # Remove any empty arrays
                if not p["other_names"]:
                    del p["other_names"]

                if not p["addresses"]:
                    del p["addresses"]

                if not p["taxonomies"]:
                    del p["taxonomies"]

                if not p["taxonomy_licenses"]:
                    del p["taxonomy_licenses"]

                if not p["licenses"]:
                    del p["licenses"]

                if not p["identifiers"]:
                    del p["identifiers"]

                if not p["taxonomy_groups"]:
                    del p["taxonomy_groups"]

            else:

                p = new_pjson_deactive_stub()
                p["number"] = row[0]
                if row[39]:
                    month = row[39][0:2]
                    day = row[39][3:5]
                    year = row[39][6:10]
                    p["basic"][
                        "deactivation_date"] = "%s-%s-%s" % (year, month, day)

            fn = "%s.json" % (p["number"])

            subdir = os.path.join(output_dir, str(p["number"])[0:4])

            try:
                os.mkdir(subdir)
            except:
                pass

            fp = os.path.join(subdir, fn)
            ofile = open(fp, 'w')
            ofile.writelines(json.dumps(p, indent=4))
            ofile.close()
            po_count += 1

            if po_count % 1000 == 0:
                pdir += 1
                out = "%s files created. Total time is %s seconds." % (
                    po_count, (time.time() - process_start_time))
                print(out)

            rowindex += 1

        if error_list:
            response_dict['num_files_created'] = rowindex - 1
            response_dict['num_file_errors'] = len(error_list)
            response_dict['errors'] = error_list
            response_dict['code'] = 400
            response_dict['message'] = "Completed with errors."
        else:

            response_dict['num_files_created'] = rowindex - 1
            response_dict['num_csv_rows'] = rowindex - 1
            response_dict['code'] = 200
            response_dict['message'] = "Completed without errors."

    return response_dict

if __name__ == "__main__":

    if len(sys.argv) != 3:
        print("Usage:")
        print("csv2pjson_public.py [CSVFILE] [OUTPUT_DIRECTORY]")
        sys.exit(1)

    csv_file = sys.argv[1]
    output_dir = sys.argv[2]

    result = publiccsv2pjson(csv_file, output_dir)

    # output the JSON transaction summary
    print(json.dumps(result, indent=4))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars - This software is public domain

from bs4 import BeautifulSoup
try:
    # For Python 3.0 and later
    from urllib.request import urlopen
except ImportError:
    # Fall back to Python 2's urllib2
    from urllib2 import urlopen
import re
import glob
import sys
from subprocess import call
from datetime import datetime


def do_update(process_full=True, download=True, delete=False):
    months = ["Jan", "Feb", "Mar", "Apr", "May", "June", "July", "Aug",
              "Sept", "Oct", "Nov", "Dec"]
    # Get just the html page
    html_page = urlopen("http://download.cms.gov/nppes/NPI_Files.html")
    link_prefix = "http://download.cms.gov/nppes/"

    soup = BeautifulSoup(html_page, "html.parser")
    month = ""
    year = datetime.now().year
    # get all links
    zipfilelinks = []
    for link in soup.findAll('a'):
        print(link)
        # get just zips
        if link.get('href', "").endswith(".zip"):
            zipfilelinks.append(link.get('href', ""))

    print(zipfilelinks)
    #determine full v/s weekly
    weeklylinks = []
    full_link = ""
    for link in zipfilelinks:

        for m in months:
            if link.__contains__(m):
                full_link = link_prefix + link
                month = m
        if link.__contains__("Report"):
            deactivation_link = link
        else:
            weeklylinks.append(link)

    # Download full file
    if process_full:

        if download:
            print("Downloading", full_link)
            call(["wget", full_link])

        # Get filename and unzip
        filename = full_link.split("/")
        zipfilename = filename[-1]
        print("Unzip", zipfilename)

        call(["unzip", zipfilename])

        # inspect the local directory for the CSV
        csv_files = glob.glob("*.csv")

        for f in csv_files:
            if f.__contains__("Header"):
                header_file = f
            else:
                main_file_to_import = f

        # Now import the file
        print("Import", main_file_to_import)
        # first convert fhir
        fhir_output_dir = "fhir-nppes-output/"
        call(["csv2fhir_public.py", main_file_to_import, fhir_output_dir])
        # Pull new PECOS files
        call(["pull_pecos.py", "y", "n", "n", "n"])
        # now upload to mongo

        call(["csv2mongo", "Base-Provider-Enrollment-File.csv", "pecos", "base", "T", "127.0.0.1", "27017"])

        call(["csv2mongo", "Reassignment-Sub-File.csv", "pecos", "reassignments", "T", "127.0.0.1", "27017"])

        call(["csv2mongo", "Addresses-Sub-File.csv", "pecos", "addresses", "T", "127.0.0.1", "27017"])



        call(["jsondir2mongo", fhir_output_dir + "/Organization/", "pecos", "fhir_organization", "T",
              "127.0.0.1", "27017"])

        call(["jsondir2mongo", fhir_output_dir + "/Practitioner/", "pecos", "fhir_practitioner", "T",
              "127.0.0.1", "27017"])


        # Index basic PECOS info
        call(["create_pecos_indexes.py", "pecos", "base", "reassignments", "addresses" "127.0.0.1",
              "27017", "Y"])


        # Make compiled pecos

        call(["makepecosdocs.py", "pecos"])

        # Index compiled pecos

        call(["create_pecos_compiled_indexes.py", "pecos", "compiled_individuals", "compiled_organizations", "127.0.0.1",
              "27017", "Y"])


        # Combine pecos and fhir indviduals/organizations and create indexes

        call(["combine_nppes_pecos_pract_fhir.py", "pecos"])

        call(["combine_nppes_pecos_org_fhir.py", "pecos"])

        call(["create_combined_indexes.py", "pecos", "fhir_practitioner", "fhir_organization", "127.0.0.1",
              "27017", "Y"])


        if delete:
            # Delete loaded files
            call(["rm -rf", fhir_output_dir])
            call(["rm -rf *.csv"])
            call(["rm -rf *.zip"])
            call(["rm -rf *.pdf"])


if __name__ == "__main__":

    if len(sys.argv) != 4:
        print("Usage:")
        print("load_pecos.py [PROCESS_FULL Y/N] "
              "[DOWNLOAD_FROM_PUBLIC_FILE Y/N] [DELETE AFTER IMPORTING TO MONGO?]")
        print("Example:")
        print("loadnppes.py Y Y Y")
        sys.exit(1)

    if sys.argv[1] in ("Y", "y", "T", True):
        process_full = True
    else:
        process_full = False

    if sys.argv[2] in ("Y", "y", "T", True):
        download = True
    else:
        download = False

    if sys.argv[3] in ("Y", "y", "T", True):
        delete = True
    else:
        delete = False

    # Get the file from the command line
    do_update(process_full, download, delete)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

# Written by Alan Viars
import json
import sys
import datetime
import re
from choices import STATES


def validate_identifier_list(l, enumeration_type):
    errors = []
    primary_count = 0
    max_values = {
        'state': 2,
        'code': 2,
        'issuer': 150,
        'identifier': 20,
    }
    for d in l:

        identifer_string = "%s %s %s issued by %s" % (d['identifier'],
                                                      d['code'], d['state'],
                                                      d['issuer'])

        for k in max_values.keys():
            if d.get(k):
                if max_values[k] < len(
                    d.get(
                        k,
                        "").encode(
                        'ascii',
                        'ignore').decode('ascii')):
                    error = "%s : %s exceeds max allowable length of %s." % (
                        identifer_string, k, max_values[k])
                    errors.append(error)

        # check for required information
        if str(
            d.get(
                'code',
                '')) not in (
            "",
            "01",
            "02",
            "04",
            "05",
            "06",
            "07",
            "08",
            "1",
            "2",
            "4",
            "5",
            "6",
            "7",
                "8"):
            error = "%s : identifier code is not in ['', '01', '02', '04','05', '06', '07', '08']" % d.get(
                'code')
            errors.append(error)

        # if state is provided then it should be valid.
        if d.get('state') and d.get('state') not in STATES:
            error = "%s : identifier code is not a valid 2-letter state code." % (
                identifer_string, d.get('state'))
            errors.append(error)

    return errors



# import unittest
# import os
# import imp
# from pdt import create_provider_indexes
#
#
# class TestCreateProviderIndexes(unittest.TestCase):
#
#     def test_ensure_provider_indexes(self):
#         """Run ensure_provider_indexes should return results """
#
#         csvfile = os.path.join(os.path.dirname(__file__),  "fiftythousand.csv")
#         outputdir = "outputdir"
#
#         result = create_provider_indexes.ensure_provider_indexes(
#             csvfile, outputdir)
#         print(result)
#
# if __name__ == '__main__':
#     unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars  - This software is public domain

import json
import sys
from baluhn import verify

from pdt.pjson.validate_basic import validate_basic_dict
from pdt.pjson.validate_addresses import validate_address_list
from pdt.pjson.validate_licenses import validate_license_list
from pdt.pjson.validate_taxonomies import validate_taxonomy_list
from pdt.pjson.validate_identifiers import validate_identifier_list
from pdt.pjson.validate_other_names import validate_other_name_list
from pdt.pjson.validate_affiliations import validate_affiliation_list

#from validate_basic import validate_basic_dict
#from validate_addresses import validate_address_list
#from validate_licenses import validate_license_list
#from validate_taxonomies import validate_taxonomy_list
#from validate_identifiers import validate_identifier_list
#from validate_other_names import validate_other_name_list
#from validate_affiliations import validate_affiliation_list

LUHN_PREFIX = "80840"


def validate_pjson(j, action):
    """
    Input a JSON object and an action. return a list of errors. If error list
    is empty then the file is valid.
    """
    action = action.lower()

    errors = []
    warnings = []
    response = {"errors": [], "warnings": []}

    # Check the action
    if action not in ("create", "update", "public"):
        error = "action must be create, update, or public."
        errors.append(error)
        response["errors"] = errors
        return response

    # Does the string contain JSON
    try:
        d = json.loads(j)
    except:
        error = "The string did not contain valid JSON."
        errors.append(error)
        response["errors"] = errors
        return response

    # Is it a dict {} (JSON object equiv)?
    if not isinstance(d, type({})):
        error = "The JSON string did not contain a JSON object i.e. {}."
        errors.append(error)
        response["errors"] = errors
        return response

    # Does it contain the top-level enumeration_type
    if "enumeration_type" not in d and action != "public":
        error = "The JSON object does not contain an enumeration_type."
        errors.append(error)
        response["errors"] = errors
        return response

    # Check for deactivation
    if "number" in d and not d.get(
        "enumeration_type",
            "") and action == "public":
        warning = "This appears to be a deactivated NPI. No information is available for deactivated NPIs."
        warnings.append(warning)
        response["warnings"] = warnings
        return response

    # Is the enumeration_type a valid?
    if d["enumeration_type"] not in ("NPI-1", "NPI-2", "OEID", "HPID"):
        error = "enumeration_type must be one of these: ('NPI-1', 'NPI-2', 'OEID', 'HPID')"
        errors.append(error)
        response["errors"] = errors
        return response

    # If a number is present we assume this is an update.
    if "number" not in d:
        number = None
    else:
        number = d['number']

    if action == "create" and number:
        warning = "enumeration number is generated by CMS. The provided value will be ignored."
        warnings.append(warning)

    if action == "update" and not number:
        error = "enumeration number is required when performaing an update."
        errors.append(error)

    # Check if the Luhn checkdigit makes is correcense.
    if number and d['enumeration_type'] in ('NPI-1', 'NPI-2'):

        prefixed_number = "%s%s" % (LUHN_PREFIX, number)
        luhn_verified = verify(prefixed_number)
        if not luhn_verified:
            warning = "The number %s did not pass Luhn algorithm check digit sanitiy check." % (
                number)
            warnings.append(warning)

    # Check for errors in the basic section
    basic_errors, basic_warnings = validate_basic_dict(d.get('basic', {}),
                                                       d.get(
                                                           'enumeration_type'),
                                                       action, number)

    # Check for errors in the addresses
    address_errors = validate_address_list(
        d.get('addresses', ()), d.get('enumeration_type'))

    # Check for errors in the license section

    if 'licenses' in d:
        license_errors = validate_license_list(
            d.get('licenses', []), d.get('enumeration_type'), action)
    else:
        license_errors = []

    taxonomy_errors = validate_taxonomy_list(
        d.get(
            'taxonomies', ()), d.get(
            'enumeration_type', ()), d.get(
                'licenses', []), d.get(
                    'taxonomy_licenses', []), d.get(
                        'basic', {}).get(
                            'sole_proprietor', "NO"), action)

    if 'identifiers' in d:
        identifier_errors = validate_identifier_list(
            d.get('identifiers', []), d.get('enumeration_type'))
    else:
        identifier_errors = []

    if 'other_names' in d:
        other_names_errors = validate_other_name_list(d.get('other_names', []),
                                                      d.get(
                                                          'enumeration_type'),
                                                      d.get('basic', {}))
    else:
        other_names_errors = []

    affiliation_errors, affiliation_warnings = validate_affiliation_list(
        d.get('affiliations', []), d.get('enumeration_type'))

    errors = errors + basic_errors + other_names_errors + address_errors + \
        license_errors + taxonomy_errors + identifier_errors + affiliation_errors
    warnings = warnings + basic_warnings
    response["errors"] = errors
    response["warnings"] = warnings
    return response


if __name__ == "__main__":

    # Get the file from the command line
    if len(sys.argv) < 3:
        print("You must supply a a ProviderJSON file to validate and an action")
        print("Usage: validate-pjson [ProivderJSON] [update|create]")
        sys.exit(1)
    else:
        pjson_file = sys.argv[1]
        action = sys.argv[2]

    if action.lower() not in ("create", "update", "public"):
        print("You must supply an action of either create or update.")
        print("Usage: validate-pjson [ProivderJSON] [update|create|public]")
        sys.exit(1)

    # Try to open the file
    try:
        fh = open(pjson_file, 'r')

        j = fh.read()

        # Validate the provider JSON content
        errors = validate_pjson(j, action)
        # Print the erros and warings as JSON to stout.
        errors_json = json.dumps(errors, indent=4)
        print(errors_json)
    except IOError:
        print("Could not open file %s." % (pjson_file))
        sys.exit(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars

import json
import sys
import datetime
import re
from choices import COUNTRIES, STATES
MIN_AGE_IN_DAYS = 6570


def validate_basic_dict(d, enumeration_type, action, number=None):
    """
    Input a python dict(d) object. Return a list of errors. If error list
    is empty then basic section is valid.
    """
    errors = []
    warnings = []
    # check values do not exceed max length

    max_values = {
        'name_prefix': 5,
        'first_name': 150,
        'last_name': 150,
        'middle_name': 150,
        'name_sufix': 5,
        'credential': 50,
        'doing_business_as': 300,
        'sole_proprieter': 3,
        'organization_name': 300,
        'organization_other_name': 300,
        'organization_other_name_code': 1,
        'ssn': 9,
        'ein': 9,
        'itin': 9,
        'gender': 1,
        'state_of_birth': 2,
        'country_of_birth': 2,
        'mode': 1,
        'status': 1,
        'contact_method': 1,
        'classification': 1,
        'deactivated_details': 1024,
        'deactivation_reason_code': 2,
        'deceased_notes': 1024,
        'parent_organization_npi': 9,
        'parent_organization_ein': 9,
        'parent_organization_legal_business_name': 300,
        'reactivation_note': 1024,
        'comments': 1024,
        'contact_person_credential': 20,
        'contact_person_email': 75,
        'contact_person_first_name': 150,
        'contact_person_last_name': 150,
        'contact_person_middle_name': 150,
        'contact_person_prefix': 5,
        'contact_person_suffix': 4,
        'contact_person_telephone_extension': 10,
        'contact_person_telephone_number': 12,
        'contact_person_title_or_position': 150,
        'authorized_official_credential': 50,
        'authorized_official_email': 75,
        'authorized_official_first_name': 300,
        'authorized_official_last_name': 300,
        'authorized_official_middle_name': 300,
        'authorized_official_prefix': 4,
        'authorized_official_suffix': 5,
        'authorized_official_telephone_number': 12,
        'authorized_official_telephone_extension': 12,
        'authorized_official_title_or_position': 300,
        'website': 1024,
        'gravatar_email': 200,
        'facebook_handle': 100,
        'twitter_handle': 100,
        'public_email': 75,
        'driving_directions': 1024,
        'bio_headline': 256,
    }

    for k in max_values.keys():
        if d.get(k):

            cleaned_value = d.get(k, "").encode(
                'ascii', 'ignore').decode('ascii')
            if max_values[k] < len(cleaned_value):
                error = "%s exceeds max allowable length of %s." % (
                    k, max_values[k])
                errors.append(error)

    # Validate Common items ------------------------------------------

    if d.get('enumeration_date'):
        try:
            date = datetime.datetime.strptime(
                d.get('enumeration_date'), '%Y-%m-%d').date()
        except ValueError:
            error = "enumeration_date must be in YYYY-MM-DD format."
            errors.append(error)
        if action != "public":
            warning = "Enumeration date is generated by CMS. The provided value will be ignored."
            warnings.append(warning)

    if d.get('last_updated'):
        try:
            date = datetime.datetime.strptime(
                d.get('last_updated'), '%Y-%m-%d').date()
        except ValueError:
            error = "last_updated must be in YYYY-MM-DD format."
            errors.append(error)

        if action != "public":
            warning = "Last updated date is generated by CMS. The provided value will be ignored."
            warnings.append(warning)

    if d.get('initial_enumeration_date'):
        try:
            date = datetime.datetime.strptime(
                d.get('initial_enumeration_date'), '%Y-%m-%d').date()
        except ValueError:
            error = "initial_enumeration_date must be in YYYY-MM-DD format."
            errors.append(error)
        if action != "public":
            warning = "Initial enumeration date is generated by CMS. The provided value will be ignored."
            warnings.append(warning)

    if d.get('date_of_death'):
        try:
            date = datetime.datetime.strptime(
                d.get('date_of_death'), '%Y-%m-%d').date()
        except ValueError:
            error = "date_of_death must be in YYYY-MM-DD format."
            errors.append(error)
        if action != "public":
            warning = "Date of death is determined by the SSA. The provided value will be ignored."
            warnings.append(warning)

    if d.get('reactivation_date'):
        try:
            date = datetime.datetime.strptime(
                d.get('reactivation_date'), '%Y-%m-%d').date()
        except ValueError:
            error = "reactivation_date must be in YYYY-MM-DD format."
            errors.append(error)

        if action != "public":
            warning = "Reactivation date is generated by CMS. the provided value will be ignored."
            warnings.append(warning)

    if d.get('deactivation_date'):
        try:
            date = datetime.datetime.strptime(
                d.get('deactivation_date'), '%Y-%m-%d').date()
        except ValueError:
            error = "deactivation_date must be in YYYY-MM-DD format."
            errors.append(error)

        if action != "public":
            warning = "Deactivation date is generated by CMS. The provided value will be ignored."
            warnings.append(warning)

    # validate phone numbers

    if d.get('contact_person_telephone_number') and not re.match(
        r'^[0-9]{3}-[0-9]{3}-[0-9]{4}$',
            d.get('contact_person_telephone_number')):
        error = "contact_person_telephone_number must be in XXX-XXX-XXXX format."
        errors.append(error)

    # Meta fields -----------------------------------
    if d.get("mode") and d.get("mode", "").upper() not in ('W', 'P', 'E', 'A'):
        # Note this should always be (E)lectronic if submitting via API
        error = "mode must be in ('W','P','E', 'A')."
        errors.append(error)

    if d.get("mode") and d.get("mode") != 'A':
        # Note this should always be (E)lectronic if submitting via API
        warning = "The mode should be set to A when using the API."
        warnings.append(warning)

    if d.get("status") and d.get(
            "status").upper() not in ('E', 'P', 'A', 'D', 'R'):

        # Note: NPPES will ignore this since status is decided by NPPES.
        error = "status must be in ('E', 'P', 'A', 'D', 'R')."
        errors.append(error)

    if d.get("status"):
        if action != "public":
            warning = "status is determined by CMS. The provided value will be ignored."
            warnings.append(warning)

    if d.get("contact_method") and d.get("contact_method") not in ('M', 'E',):
        if action != "public":
            # Note: NPPES will ignore this since status is decided by NPPES.
            error = "contact_method must be in ('M', 'E')."
            errors.append(error)

    # NPI-1 -------------------------------------------------------------------
    if enumeration_type == "NPI-1":
        # Ensure required fields for NPI-1

        if not d.get('first_name'):
            error = "first_name is required."
            errors.append(error)

        if not d.get('last_name'):
            error = "last_name is required."
            errors.append(error)

        if d.get("name_suffix") and d.get("name_suffix").upper() not in (
                'JR.', 'SR.', 'I', 'II', 'III', 'IV', 'V', 'VI', 'VII', 'VIII', 'IX', 'X'):
            error = """name_suffix must be in ['Jr.', 'Sr.', 'I', 'II', 'III', 'IV', 'V', 'VI', 'VII', 'VIII', 'IX', 'X']'
            """
            errors.append(error)

        if not d.get('sole_proprietor'):
            error = "sole_proprietor is required and must be in ('YES', 'NO')."
            errors.append(error)
        else:
            if d.get('sole_proprietor').upper() not in ("YES", "NO"):
                error = "sole_proprietor must be in ('YES', 'NO')."
                errors.append(error)
        # Gender
        if action == "create" and not d.get('gender'):
            error = "gender is required."
            errors.append(error)

        elif action == "create" and d.get('gender'):
            if d.get('gender') not in ("M", "F", "T"):
                error = "gender must be in ('F','M', 'T')."
                errors.append(error)

        elif action == "update" and d.get('gender'):
            warning = "gender cannot be updated via API. Please contact the Enumerator help desk. The provided value will be ignored."
            warnings.append(warning)

        # Date of Birth
        if action == "create" and not d.get('date_of_birth'):
            error = "date_of_birth is required when creating a new individual's record."
            errors.append(error)

        elif action == "update" and d.get('date_of_birth'):
            warning = "date_of_birth cannot be updated via API. Please contact the Enumerator help desk. The provided value will be ignored."
            warnings.append(warning)

        elif action == "create" and d.get('date_of_birth'):
            # date supplied so let's make sure it is valid
            try:
                date_of_birth = datetime.datetime.strptime(
                    d.get('date_of_birth'), '%Y-%m-%d').date()

                age_days = datetime.date.today() - date_of_birth
                if age_days.days < MIN_AGE_IN_DAYS:
                    errors.append(
                        "Candidate must be at least 16 years of age.")

            except ValueError:
                error = "date_of_birth must be in YYYY-MM-DD format."
                errors.append(error)

        # State of Birth
        if action == "create" and not d.get('state_of_birth'):
            error = "state_of_birth is required. Use ZZ if born outside the US."
            errors.append(error)
        elif action == "create" and d.get('state_of_birth'):
            if d.get('state_of_birth') not in STATES:
                error = "state_of_birth must be 2 letter ISO code or ZZ for foreign born."
                errors.append(error)
        elif action == "update" and d.get('state_of_birth'):
            warning = "state_of_birth cannot be updated via API. Please contact the Enumerator help desk. The provided value will be ignored."
            warnings.append(warning)

        # Country of Birth
        if action == "create" and not d.get('country_of_birth'):
            error = "country_of_birth is required for NPI-1 creation.."
            errors.append(error)
        elif action == "create" and d.get('country_of_birth'):
            if d.get('country_of_birth') not in COUNTRIES:
                error = "country_of_birth must be 2 letter ISO code."
                errors.append(error)
        elif action == "update" and d.get('country_of_birth'):
            warning = "country_of_birth cannot be updated via API. Please contact the Enumerator help desk. The provided value will be ignored."
            warnings.append(warning)

        # Validate the interdependecies
        if d.get('country_of_birth') and d.get('country_of_birth') and (
                d.get('country_of_birth')) != "US" and (
                d.get('state_of_birth') != "ZZ"):
            error = """country_of_birth and state_of_birth mismatch. A person cannot be born in both a foreign contry and a US state at the same time."""
            errors.append(error)

        if action == "create" and not d.get('ssn') and not d.get('itin'):
            error = "An NPI-1 individual provider must supppy either an SSN or an EIN."
            errors.append(error)

        if action == "update" and d.get('ssn'):
            warning = "Social security number (ssn) cannot be updated via API. Please contact the Enumerator help desk. The provided value will be ignored."
            warnings.append(warning)

        if action == "update" and d.get('itin'):
            warning = "Individual Taxpayer Identification Number (itin) cannot be updated via API. Please contact the Enumerator help desk. The provided value will be ignored."
            warnings.append(warning)

        if d.get('ssn') and len(str(d.get('ssn'))) != 9:
            error = "SSN must be 9 digits."
            errors.append(error)

        if d.get('itin') and len(str(d.get('itin'))) != 9:
            error = "ITIN must be 9 digits."
            errors.append(error)

        # ensure a contact person is given
        if not d.get('contact_person_email'):
            if action != "public":
                error = "contact_person_email must be provided."
                errors.append(error)

        # ensure a contact person is given
        if not d.get('contact_person_first_name'):
            if action != "public":
                error = "contact_person_first_name must be provided."
                errors.append(error)

        # ensure a contact person is given
        if not d.get('contact_person_last_name'):
            if action != "public":
                error = "contact_person_last_name must be provided."
                errors.append(error)

        # ensure a contact person is given
        if not d.get('contact_person_telephone_number'):
            if action != "public":
                error = "contact_person_telephone_number must be provided."
                errors.append(error)

        # Validate the not required items NPI-1

        if d.get("name_prefix") and d.get("name_prefix").upper() not in (
                'MS.', 'MR.', 'MISS', 'MRS.', 'DR.', 'PROF.'):
            error = "name_prefix  must be one of the following: 'MS.', 'MR.', 'MISS', 'MRS.', 'DR.', 'PROF.'"
            errors.append(error)

    if enumeration_type == "NPI-2":
        # Validate the organization
        if not d.get('organization_name', ""):
            error = "organization_name is required for a type 2 (organization) NPI."
            errors.append(error)
        else:
            if len(d.get('organization_name')) > 300:
                error = "organization_name is longer than allowed."
                errors.append(error)

        if action != "public":
            if not d.get('ein'):
                error = "EIN is required for a type-2 organization provider."
                errors.append(error)

        if d.get('ein') and len(str(d.get('ein'))) != 9:
            error = "EIN must be 9 digits."
            errors.append(error)

        if d.get('authorized_official_telephone_number') and not re.match(
                r'^[0-9]{3}-[0-9]{3}-[0-9]{4}$', d.get('authorized_official_telephone_number')):
            error = "authorized_official_telephone_number must be in XXX-XXX-XXXX format."
            errors.append(error)

        if not d.get('authorized_official_first_name'):
            error = "authorized_official_first_name is required for a type-2 organization provider."
            errors.append(error)

        if not d.get('authorized_official_last_name'):
            error = "authorized_official_last_name is required for a type-2 organization provider."
            errors.append(error)

        if not d.get('authorized_official_title_or_position'):
            error = "authorized_official_title_or_position is required for a type-2 organization provider."
            errors.append(error)

        if not d.get('authorized_official_telephone_number'):
            error = "authorized_official_telephone_number is required for a type-2 organization provider."
            errors.append(error)
    retval = [errors, warnings]
    return retval



import unittest
import os
from pdt import vnpi


class Testvnpi(unittest.TestCase):

    def test_vnpi(self):
        """Run a real npi and a fake npi to see if True and False """
        self.assertTrue(vnpi.verify_npi(1679576722))
        self.assertFalse(vnpi.verify_npi(1111111111))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

# Written by Alan Viars
import json
import sys
import datetime
import re
from choices import (
    SUFFIX_CHOICES,
    PREFIX_CHOICES,
    INDIVIDUAL_OTHER_NAME_CHOICES,
    SOLE_PROPRIETOR_OTHER_NAME_CHOICES,
    ORGANIZATION_OTHER_NAME_CHOICES,
    OTHER_NAME_CHOICES)


def validate_other_name_list(l, enumeration_type, basic):
    errors = []
    primary_count = 0

    if enumeration_type == "NPI-1":
        sole_proprietor = basic['sole_proprietor']
    else:
        sole_proprietor = "NO"

    max_values = {
        'type': 100,
        'code': 1,
        'prefix': 5,
        'organization_name': 150,
        'first_name': 150,
        'middle_name': 150,
        'last_name': 150,
        'suffix': 4,
        'credential': 50,
        'othertype': ''
    }
    i = 0
    for d in l:

        for k in max_values.keys():
            if d.get(k):
                if max_values[k] < len(
                    d.get(
                        k,
                        "").encode(
                        'ascii',
                        'ignore').decode('ascii')):
                    error = "other_name %s: %s exceeds max allowable length of %s." % (
                        i, k, max_values[k])
                    errors.append(error)

        # Type II
        if enumeration_type == "NPI-2" and str(d.get('code',
                                                     '')) not in ORGANIZATION_OTHER_NAME_CHOICES:
            error = "other_name %s: %s other_name for organizations is not in %s." % (
                i, d.get('code'), ORGANIZATION_OTHER_NAME_CHOICES)
            errors.append(error)

        if enumeration_type == "NPI-2" and not d.get('organization_name', ''):
            error = "other_name %s: organization_name required for NPI-2," % (
                i)
            errors.append(error)

        # Type 1
        if enumeration_type == "NPI-1" and sole_proprietor == "NO" and str(
                d.get('code', '')) not in INDIVIDUAL_OTHER_NAME_CHOICES:
            error = "other_name %s: %s other_name for individuals is not in %s" % (
                i, d.get('code'), INDIVIDUAL_OTHER_NAME_CHOICES)
            errors.append(error)

        if enumeration_type == "NPI-1" and sole_proprietor == "NO" and not str(
                d.get('first_name', '')):
            error = "other_name %s: first_name required for NPI-1." % (i)
            errors.append(error)

        if enumeration_type == "NPI-1" and sole_proprietor == "NO" and not str(
                d.get('last_name', '')):
            error = "other_name %s: last_name required for NPI-2" % (i)
            errors.append(error)

        # Type 1 Sole proprietor
        if enumeration_type == "NPI-1" and sole_proprietor == "YES" and str(
                d.get('code', '')) not in SOLE_PROPRIETOR_OTHER_NAME_CHOICES:
            error = "other_name %s: %s other_name for sole_proprietor is not in %s." % (
                i, d.get('code'), SOLE_PROPRIETOR_OTHER_NAME_CHOICES)
            errors.append(error)

        if enumeration_type == "NPI-1" and sole_proprietor ==  "YES" \
                and not ( str(d.get('last_name', '')) and str(d.get('first_name', ''))) \
                and not str(d.get('organization_name', '')):
            error = "other_name %s: Either organization_name or first_name and last_name are required for a sole proprietor." % (
                i)
            errors.append(error)

        # General
        # supply othertypwe when type is not given.
        if not str(d.get('code', '')) and not str(d.get('othertype', '')):
            error = "other_name %s: othertype must be supplied if other_name type code is blank." % (
                i)
            errors.append(error)

        # suffix
        suffix = d.get('suffix', '')
        if suffix and suffix not in SUFFIX_CHOICES:
            error = "other_name %s: suffix must be in %s" % (i, SUFFIX_CHOICES)
            errors.append(error)

        # prefix
        prefix = d.get('prefix', '')
        if prefix and prefix not in PREFIX_CHOICES:
            error = "other_name %s: prefix must be in %s" % (i, PREFIX_CHOICES)
            errors.append(error)

        i += 1
    return errors



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars

import json
import sys
import os
import datetime
import csv


def validate_taxonomy_list(taxonomies, enumeration_type, licenses,
                           taxonomy_licenses, sole_proprietor, action):

    csvfile = os.path.join(os.path.dirname(__file__),
                           "taxonomy-license-crosswalk.csv")

    errors = []
    primary_count = 0

    taxonomy_license_crosswalk = {}
    # Load the CSV
    fh = open(csvfile, 'rb')
    csvr = csv.reader(fh, delimiter=',')
    for row in csvr:
        key = row[0]
        entity_type = row[1]
        if row[2] == "False":
            license_required = False
        else:
            license_required = True

        taxonomy_license_crosswalk[key] = (entity_type, license_required)

    for t in taxonomies:
        if action != "public":

            # check for required information
            if t.get('code') not in taxonomy_license_crosswalk.keys():
                error = "%s : code (taxonomy) is not a valid taxonomy code. See http://www.wpc-edi.com/taxonomy" % t.get('code')
                errors.append(error)

        if not isinstance(t.get('primary'), type(True)):
            error = "%s : primay must be true or false." % (d.get('code'))
            errors.append(error)

        if t.get('primary'):
            primary_count += 1
    # check that only one taxonomy is marked as primary
    if primary_count != 1:
        error = "Exactly 1 taxonomy code must be marked as primary. The primary count is %s." % (
            primary_count)
        errors.append(error)

    # Get the taxonomy license codes
    tl_codes = []
    for tl in taxonomy_licenses:
        tl_codes.append(tl.get('taxonomy_code'))

    for t in taxonomies:
        # print t
        # check for required information

        # Check to see if the taxonomy is right for the enumeration_type
        if t.get('code') in taxonomy_license_crosswalk.keys():
            cw = taxonomy_license_crosswalk[t.get('code')]
            code = cw[0]
            requires_license = cw[1]

            # print code , requires_license
            if action != "public":
                if cw[
                        0] == "I" and enumeration_type != "NPI-1" and sole_proprietor == "NO":
                    error = "Taxonomy %s is only for for individuals / NPI-1." % (
                        t.get('code'))
                    errors.append(error)
                if cw[0] == "O" and enumeration_type != "NPI-2":
                    error = "Taxonomy %s is only for for organizations / NPI-2." % (
                        t.get('code'))
                    errors.append(error)
                # Check to see if the taxonomy requires a license
                if requires_license and t.get('code') not in tl_codes:
                    error = "Taxonomy %s requires a license." % (t.get('code'))
                    errors.append(error)

    return errors



import unittest
import os
import random
import pdt.json_schema_check
from pdt import csv2fhir_public


class TestCSV2FHIR_PUBLIC(unittest.TestCase):

    def test_run_CSV2FHIR_PUBLIC(self):
        """Run CSV2FHIR should return results """
        CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))
        csvfile = os.path.join(CURRENT_DIR, "fiftythousand.csv")
        output_dir = "fhir_test_output"
        with open(csvfile):

            result = csv2fhir_public.publiccsv2fhir(csvfile, output_dir)
            self.assertTrue(os.path.exists("fhir_test_output"))

    def test_FHIR_results(self):
        """Put newly created files through json_schema to check"""
        CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))

        for x in range(1000, 2000):
            if os.path.exists(os.path.join("fhir_test_output", "Practitioner",
                                           str(x))):
                for files in os.listdir(os.path.join("fhir_test_output",
                                                     "Practitioner", str(x))):
                    result = pdt.json_schema_check.json_schema_check(
                        os.path.join(CURRENT_DIR, "..", "pdt",
                                     "fhir_json_schema", "Practitioner.json"),
                        os.path.join("fhir_test_output", "Practitioner",
                                     str(x), files))
                    self.assertEqual(result['errors'], [])
                # To test to make sure works when running from different
                # directories, using nosetest.
                # If not using a test runner, make sure to run tests from the
                # project root.
                # if x > 1200:
                #     self.fail()
                    # print(x)
            if os.path.exists(os.path.join("fhir_test_output", "Organization",
                                           str(x))):
                for files in os.listdir(os.path.join("fhir_test_output",
                                                     "Organization", str(x))):
                    result = pdt.json_schema_check.json_schema_check(
                        os.path.join(CURRENT_DIR, "..", "pdt",
                                     "fhir_json_schema", "Organization.json"),
                        os.path.join("fhir_test_output", "Organization",
                                     str(x), files))
                    self.assertEqual(result['errors'], [])
                # if x > 1200:
                #     self.fail()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars - This software is public domain

import sys
from datetime import datetime

from pymongo import MongoClient
import os
import sys
import string
import json
import csv
from collections import OrderedDict


MONGO_HOST = "127.0.0.1"
MONGO_PORT = 27017


def makepecos_fhir_db(database_name="pecos"):

    i = 0
    # try:

    mc = MongoClient(host=MONGO_HOST, port=MONGO_PORT)
    db = mc[database_name]
    base_collection = db['base']
    addresses_collection = db['addresses']
    reassignments_collection = db['reassignments']
    compiled_fhir_individuals_collection = db['compiled_individuals_fhir']
    compiled_fhir_organizations_collection = db['compiled_organizations_fhir']

    for bdoc in base_collection.find():
        i += 1
        d = OrderedDict()
        if bdoc['ENRLMT_ID'].startswith('O'):
            d['resourceType'] = "Organization"
        elif bdoc['ENRLMT_ID'].startswith('I'):
            d['resourceType'] = "Practitioner"

        # Get name
        name = OrderedDict()
        if bdoc['FIRST_NAME']:

            name['family'] = [bdoc['LAST_NAME']]
            name['given'] = [bdoc['FIRST_NAME']]

            d['name'] = [name]
        else:
            d['name'] = bdoc['ORG_NAME']

        # Description Extension
        # **extension for entire stript **
        extension = []
        description = OrderedDict()
        description['url'] = 'https://data.cms.gov/public-provider-enrollment'
        description['valueString'] = bdoc['PROVIDER_TYPE_DESC']
        extension.append(description)

        # NPI
        identifier = OrderedDict()
        identifier['value'] = bdoc['NPI']
        identifier['use'] = 'official'
        identifier['system'] = 'http://hl7.org/fhir/sid/us-npi'
        d['identifier'] = [identifier]

        extension_works_for = []
        extension_has_providers = []
        for rdoc in reassignments_collection.find(
                {"REASGN_BNFT_ENRLMT_ID": bdoc['ENRLMT_ID']}):

            ac = base_collection.find(
                {'ENRLMT_ID': rdoc["RCV_BNFT_ENRLMT_ID"]})
            for a in ac:
                # Create Codings
                npi_coding = OrderedDict()
                npi_coding[
                    'system'] = 'https://nppes.cms.hhs.gov/NPPES/Welcome.do'
                npi_coding['code'] = a['NPI']
                npi_coding['display'] = 'NPI number of affiliation'
                # Leaving off the 'userSelected' category for now.

                enrollmentid_coding = OrderedDict()
                enrollmentid_coding[
                    'system'] = 'https://data.cms.gov/public-provider-enrollment'
                enrollmentid_coding['code'] = a['ENRLMT_ID']
                enrollmentid_coding[
                    'display'] = 'PECOS Enrollment ID of affiliation'

                # Create Codeable concept
                value_codeable_concept = OrderedDict()
                value_codeable_concept['coding'] = [
                    npi_coding, enrollmentid_coding]
                if d['resourceType'] == "Organization":
                    value_codeable_concept['text'] = a[
                        'ORG_NAME'] + ',' + a['PROVIDER_TYPE_DESC']
                elif d['resourceType'] == "Practitioner":
                    value_codeable_concept['text'] = a[
                        'FIRST_NAME'] + ',' + a['LAST_NAME'] + ',' + a['PROVIDER_TYPE_DESC']

                # Create fhir affiliation from compiled_individuals
                affiliation = OrderedDict()
                affiliation[
                    'url'] = 'https://data.cms.gov/public-provider-enrollment'
                # print(value_codeable_concept['text'])
                affiliation['valueCodeableConcept'] = value_codeable_concept
                # wrap in list
                # affiliation = [affiliation]
                extension_works_for.append(affiliation)

        for rdoc in reassignments_collection.find(
                {"RCV_BNFT_ENRLMT_ID": bdoc['ENRLMT_ID']}):

            ac = base_collection.find(
                {'ENRLMT_ID': rdoc["REASGN_BNFT_ENRLMT_ID"]})
            for a in ac:
                # Create Codings
                npi_coding = OrderedDict()
                npi_coding[
                    'system'] = 'https://nppes.cms.hhs.gov/NPPES/Welcome.do'
                npi_coding['code'] = a['NPI']
                npi_coding['display'] = 'NPI number of affiliation'
                # Leaving off the 'userSelected' category for now.

                enrollmentid_coding = OrderedDict()
                enrollmentid_coding[
                    'system'] = 'https://data.cms.gov/public-provider-enrollment'
                enrollmentid_coding['code'] = a['ENRLMT_ID']
                enrollmentid_coding[
                    'display'] = 'PECOS Enrollment ID of affiliation'

                # Create Codeable concept
                value_codeable_concept = OrderedDict()
                value_codeable_concept['coding'] = [
                    npi_coding, enrollmentid_coding]
                if d['resourceType'] == "Organization":
                    value_codeable_concept['text'] = a[
                        'ORG_NAME'] + ',' + a['PROVIDER_TYPE_DESC']
                elif d['resourceType'] == "Practitioner":
                    value_codeable_concept['text'] = a[
                        'FIRST_NAME'] + ',' + a['LAST_NAME'] + ',' + a['PROVIDER_TYPE_DESC']

                # Create fhir affiliation from compiled_individuals
                affiliation = OrderedDict()
                affiliation[
                    'url'] = 'https://data.cms.gov/public-provider-enrollment'
                # print(value_codeable_concept['text'])
                affiliation['valueCodeableConcept'] = value_codeable_concept
                # wrap in list
                affiliation = affiliation
                extension_has_providers.append(affiliation)

        if extension_has_providers == []:
            # del d["extension_has_providers"]
            extension.append(extension_works_for)

        if extension_works_for == []:
            # del d["extension_works_for"]
            extension.append(extension_has_providers)
        d['extension'] = extension

        if d['resourceType'] == "Organization":
            compiled_fhir_organizations_collection.insert(d)
        elif d['resourceType'] == "Practitioner":
            compiled_fhir_individuals_collection.insert(d)

        # print json.dumps(d, indent =4)

    # Walk through base

    # except:
    #     print(sys.exc_info)

    print(i, "Processed")


if __name__ == "__main__":

    if len(sys.argv) != 2:
        print("Usage:")
        print("makepecosdocs.py [DATABASE NAME]")
        sys.exit(1)

    database_name = sys.argv[1]
    # Run it
    makepecos_fhir_db(database_name=database_name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars - This software is public domain

import os
import sys
import json
import jsonschema
from jsonschema import exceptions
from collections import OrderedDict


def json_schema_check_fhir(fhir_key_word_schema, json_string):

    input_json_string = json.loads(json_string)
    fhir_json_schema = os.path.join(os.path.dirname(
        __file__), 'fhir_json_schema', '%s.json' % (fhir_key_word_schema))

    with open(fhir_json_schema) as json_schema:
        deserialized_json_schema = json.load(json_schema)

    v = jsonschema.Draft4Validator(deserialized_json_schema)
    errors = sorted(v.iter_errors(input_json_string), key=lambda e: e.path)
    for error in errors:
        print(error.message)

    """JSON SCHEMA CHECK"""
    results = OrderedDict()
    if errors:
        results['errors'] = errors

    else:
        results['errors'] = []
        results[
            'results'] = "Congrats! The JSON file fits the schema. There were no errors."

    return results

if __name__ == "__main__":

    if len(sys.argv) != 3:
        print("Usage:")
        print("json_schema_check_fhir.py [FHIR JSON SCHEMA] [JSON STRING]")
        sys.exit(1)

    fhir_key_word_schema = sys.argv[1]
    json_string = sys.argv[2]

    result = json_schema_check_fhir(fhir_key_word_schema, json_string)
    print(json.dumps(result, indent=4))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars - This software is public domain

from bs4 import BeautifulSoup
try:
    # For Python 3.0 and later
    from urllib.request import urlopen
except ImportError:
    # Fall back to Python 2's urllib2
    from urllib2 import urlopen
import re
import glob
import sys
from subprocess import call
from datetime import datetime


def do_update(process_full=True, download=True, delete=False):
    json_output_dir = "./json-output"
    fhir_output_dir = "./fhir-output"

    months = ["January", "Feb", "March", "April", "May", "June",
              "July", "August", "September", "Oct", "November", "December"]
    # Get just the html page
    html_page = urlopen("http://download.cms.gov/nppes/NPI_Files.html")
    link_prefix = "http://download.cms.gov/nppes/"

    soup = BeautifulSoup(html_page, "html.parser")
    month = ""
    year = datetime.now().year
    # get all links
    zipfilelinks = []
    for link in soup.findAll('a'):
        # print(link)
        # get just zips
        if link.get('href', "").endswith(".zip"):
            zipfilelinks.append(link.get('href', ""))
    print(zipfilelinks)
    # determine full v/s weekly
    weeklylinks = []
    full_link = ""
    for link in zipfilelinks:

        for m in months:
            if link.__contains__(m):
                full_link = link_prefix + link
                month = m
        if link.__contains__("Report"):
            deactivation_link = link

        else:
            weeklylinks.append(link)

    # Download full file
    if process_full:
        filename = full_link
    else:
        filename = link_prefix + weeklylinks[-1]

    print(filename)
    if download:
        print("Downloading", filename)
        call(["wget", filename])

    # Get filename and unzip

    zipfilename = 'NPPES*.zip'
    print("Unzip", zipfilename)

    call(["unzip", zipfilename])

    # inspect the local directory for the CSV
    csv_files = glob.glob("*.csv")

    for f in csv_files:
        if f.__contains__("Header"):
            header_file = f
        else:
            main_file_to_import = f
            # Now import the file
            print("Import", main_file_to_import)

    # first convert to json
    json_output_dir = "json-output"
    fhir_output_dir = "fhir-output"
    call(["csv2pjson_public.py", main_file_to_import, json_output_dir])
    call(["csv2fhir_public.py", main_file_to_import, fhir_output_dir])

    # now upload to mongo
    call(["jsondir2mongo", json_output_dir, "nppes",
          "pjson", "T", "127.0.0.1", "27017"])
    call(["jsondir2mongo", fhir_output_dir + "/Practitioner",
          "nppes", "fhir_practitioner", "T", "127.0.0.1", "27017"])
    call(["jsondir2mongo", fhir_output_dir + "/Organization",
          "nppes", "fhir_organization", "T", "127.0.0.1", "27017"])

    # now create indexes
    call(["create_provider_indexes.py", "nppes", "pjson",
          "fhir_practitioner", "fhir_organization", "127.0.0.1", "27017", "Y"])

    if delete:
        # Delete loaded files
        call(["rm -rf", json_output_dir])
        call(["rm -rf", fhir_output_dir])
        call(["rm -rf *.csv"])
        call(["rm -rf *.zip"])
        call(["rm -rf *.pdf"])


if __name__ == "__main__":

    if len(sys.argv) != 4:
        print("Usage:")
        print(
            "loadnppes.py [PROCESS_FULL Y/N] [DOWNLOAD_FROM_PUBLIC_FILE Y/N] [DELETE FILES AFTER UPLOADED TO MONGO?]")
        print("Example:")
        print("loadnppes.py Y Y Y")
        sys.exit(1)

    if sys.argv[1] in ("Y", "y", "T", True):
        process_full = True
    else:
        process_full = False

    if sys.argv[2] in ("Y", "y", "T", True):
        download = True
    else:
        download = False

    if sys.argv[3] in ("Y", "y", "T", True):
        delete = True
    else:
        delete = False

    # Get the file from the command line
    do_update(process_full, download, delete)



import unittest
import os
from pdt import csv2pjson_public


class TestCSV2PJSONPublic(unittest.TestCase):

    # Be more specific about the dictionary that is created

    def test_pjson_deactivation(self):
        csv2pjson_public.new_pjson_deactive_stub

    def test_pjson_creation(self):
        csv2pjson_public.new_pjson_stub

    def test_publiccsv2pjson(self):
        """Run CSV2pjson_public should return results """

        csvfile = os.path.join(os.path.dirname(__file__),  "fiftythousand.csv")
        output_dir = "test_output"
        number_processed = 50000

        result = csv2pjson_public.publiccsv2pjson(csvfile, output_dir)
        self.assertEqual(result['num_files_created'], number_processed)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars

import json
import sys
import datetime
import re
from choices import AFFILIATION_DATA_TYPE, AFFILIATION_PURPOSE, ENDPOINT_DATA_TYPE, STATES
from validate_email import validate_email
from baluhn import verify
LUHN_PREFIX = "80840"


def validate_affiliation_list(l, enumeration_type):
    errors = []
    warnings = []
    primary_count = 0
    max_values = {
        'purpose': 25,
        'affiliation_data_type': 10,
        'endpoint_data_type': 50,
        'affiliation_identifier': 1024,
        'endpoint': 1024,
        'description': 1024,
        'state': 2,
    }

    i = 0

    for d in l:

        affiliation_string = "Affiliation %s: " % (i)

        for k in max_values.keys():
            if d.get(k):
                if max_values[k] < len(
                    d.get(
                        k,
                        "").encode(
                        'ascii',
                        'ignore').decode('ascii')):
                    error = "%s : %s exceeds max allowable length of %s." % (
                        affiliation_string, k, max_values[k])
                    errors.append(error)

        # check for required information
        if d.get('purpose') not in AFFILIATION_PURPOSE:
            error = "%s purpose %s is not a valid value.  Valid values are %s." % (
                affiliation_string, d.get('purpose'), AFFILIATION_PURPOSE)
            errors.append(error)

        if d.get('affiliation_data_type') not in AFFILIATION_DATA_TYPE:
            error = "%s affiliation_data_type %s is not a valid value.  Valid values are %s." % (
                affiliation_string, d.get('affiliation_data_type'), AFFILIATION_DATA_TYPE)
            errors.append(error)

        if not str(d.get('affiliation_data_type')) and \
                d.get('purpose') not in AFFILIATION_PURPOSE:
            error = "%s affiliation_data_type %s is required when purpose is one of these %s" % (
                affiliation_string, d.get('affiliation_data_type'), AFFILIATION_PURPOSE)
            errors.append(error)

        if d.get('endpoint_data_type') and d.get(
                'endpoint_data_type') not in ENDPOINT_DATA_TYPE:
            error = "%s endpoint_data_type %s is not a valid value. Valid values are %s." % (
                affiliation_string, d.get('endpoint_data_type', ''), ENDPOINT_DATA_TYPE)
            errors.append(error)

        if d.get('purpose',
                 '') == "HIE-EXCHANGE" and not d.get('endpoint_data_type',
                                                     ''):
            error = "%s endpoint_data_type is required when the purpose is HIE-EXCHANGE." % \
                (affiliation_string)
            errors.append(error)

        if not d.get('affiliation_identifier', '') and \
                d.get('affiliation_data_type', '') in AFFILIATION_DATA_TYPE:
            error = "%s affiliation_identifier is required when affiliation_data_type is in %s." % (
                affiliation_string, AFFILIATION_DATA_TYPE)
            errors.append(error)

        if not d.get('endpoint') and d.get(
                'purpose') in ("HIE-EXCHANGE", "DOMAIN"):
            error = "%s endpoint is required when purpose is in %s." % \
                (affiliation_string, ("HIE-EXCHANGE", "DOMAIN"))
            errors.append(error)

        if d.get('accepting_new_patients', None) not in (True, False, None):
            error = "%s accepting_new_patients must be boolean. i.e. true or false." % (
                affiliation_string)
            errors.append(error)

        if d.get(
                'for_additional_documentation_request',
                None) not in (
                True,
                False,
                None):
            error = "%s for_additional_documentation_request must be boolean. i.e. true or false." % (
                affiliation_string)
            errors.append(error)

        if d.get('purpose') == "MEDICAID-NETWORK" and not d.get('state'):
            error = "%s state is required when purpose = MEDICIAD-NETWORK." % \
                (affiliation_string)
            errors.append(error)

        if d.get('state') and d.get('state') not in STATES:
            error = "%s state %s is not a valid value. Valid values are %s." % (
                affiliation_string, d.get('state'), STATES)
            errors.append(error)
        if d.get('affiliation_data_type') in ('NPI-1', 'NPI-2'):
            prefixed_number = "%s%s" % (
                LUHN_PREFIX, d['affiliation_identifier'])
            luhn_verified = verify(prefixed_number)
            if not luhn_verified:
                error = "The NPI affiliation_identifier %s did not pass Luhn algorithm check digit sanitiy check." % (d[
                                                                                                                      'affiliation_identifier'])
                errors.append(error)

        if d.get('endpoint_data_type') in (
            'DIRECT-EMAIL-ADDRESS',
                'REGULAR-EMAIL-ADDRESS'):
            is_valid = validate_email(d.get('endpoint'))
            if not is_valid:
                error = "%s %s has and endpoint_data_type of %s and is not a valid email." % (
                    affiliation_string, d.get('endpoint'), d.get('endpoint_data_type'))
                errors.append(error)

        i += 1
    retval = [errors, warnings]
    return retval



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars

import json
import sys
import datetime
import re
from choices import COUNTRIES, STATES, ADDRESS_PURPOSE, ADDRESS_TYPE


def validate_address_list(l, enumeration_type):
    errors = []
    LOCATION_ADDRESS_FOUND = False
    MAILING_ADDRESS_FOUND = False
    # define a max_values dict
    max_values = {
        'address_type': 12,
        'address_purpose': 20,
        'address_1': 200,
        'address_2': 200,
        'city': 200,
        'state': 2,
        'zip': 10,
        'country_code': 2,
        'us_telephone_number': 12,
        'telephone_number_extension': 10,
        'us_fax_number': 12,
        'foreign_telephone_number': 20,
        'foreign_fax_number': 20,
        'mpo': 3,
        'lat': 20,
        'long': 20,
    }

    # Test for max_length errors

    for d in l:
        address_string = "%s %s %s %s %s" % (d.get('address_1', "").encode('ascii', 'ignore').decode('ascii'),
                                             d.get('address_2', "").encode(
                                                 'ascii', 'ignore').decode('ascii'),
                                             d.get('city', "").encode(
                                                 'ascii', 'ignore').decode('ascii'),
                                             d.get('state', "").encode(
                                                 'ascii', 'ignore').decode('ascii'),
                                             d.get('zip', "").encode('ascii', 'ignore').decode('ascii'))

        for k in max_values.keys():
            if d.get(k):
                if max_values[k] < len(d.get(k, "").encode('ascii', 'ignore').decode('ascii')):

                    error = "%s : %s max allowable length %s." % (
                        address_string, k, max_values[k])
                    errors.append(error)

        # check for required information

        if d.get('address_type') not in ADDRESS_TYPE:
            error = "%s : address_type must be in %s." % (
                address_string, ADDRESS_TYPE)
            errors.append(error)

        if d.get('override_address_standardization') and \
           d.get('override_address_standardization') not in (True, False):
            error = "%s : override_address_standardization must be true or false." % (
                address_string)
            errors.append(error)

        if d.get('accept_address_standardization') and \
           d.get('accept_address_standardization') not in (True, False):
            error = "%s : accept_address_standardization must be true or false." % (
                address_string)
            errors.append(error)

        if d.get('address_purpose') not in ADDRESS_PURPOSE:
            error = "%s : address_purpose must be in %s." % (
                address_string, ADDRESS_PURPOSE)
            errors.append(error)

        if d.get('address_purpose') == "MAILING":
            MAILING_ADDRESS_FOUND = True

        if d.get('address_purpose') == "LOCATION":
            LOCATION_ADDRESS_FOUND = True

        if not d.get('address_1'):
            error = "%s : address_1 is required." % (address_string)
            errors.append(error)

        if not d.get('city'):
            error = "%s : city is required." % (address_string)
            errors.append(error)

        if not d.get('state'):
            error = "%s : state is required." % (address_string)
            errors.append(error)

        if not d.get('zip'):
            error = "%s : zip is required." % (address_string)
            errors.append(error)

        if not d.get('country_code'):
            error = "%s : country_code is required." % (address_string)
            errors.append(error)

        if d.get('state') not in STATES:
            error = "%s : state must be 2 letter ISO code or set to ZZ for a foreign country." % (
                address_string)
            errors.append(error)

        if d.get('country_code') not in COUNTRIES:
            error = "%s : country_code must be 2 letter ISO code." % (
                address_string)
            errors.append(error)

    # Check to see if both a mail and practice location is on file for NPI-1
    # and NPI-2
    if enumeration_type in ("NPI-1", "NPI-2") and not LOCATION_ADDRESS_FOUND:
        errors.append("A practice location address was not found.")

    if enumeration_type in ("NPI-1", "NPI-2", "HPID", "OEID") and not MAILING_ADDRESS_FOUND:
        errors.append("A mailing address was not found.")

    return errors



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

# Written by Alan Viars
import json
import sys
import datetime
import re
from choices import LICENSE_TYPE_CODES


def validate_license_list(l, enumeration_type, action):
    errors = []

    # define a max_values dict
    max_values = {
        'code': 50,
        'status': 10,
    }

    # Test for max_length errors

    for d in l:
        # form an address string.

        for k in max_values.keys():
            if d.get(k):
                if max_values[k] < len(
                    d.get(
                        k,
                        "").encode(
                        'ascii',
                        'ignore').decode('ascii')):
                    error = "%s : %s max allowable length %s." % (
                        d.get['code'], max_values[k])
                    errors.append(error)

        if len(d.get('code')) < 7:
            # The code is to shourt to be valid
            error = "%s : License code is too short to be valid." % (
                d.get('code'))
            errors.append(error)
        else:
            license_code = d.get('code')[0:6]
            if action != "public":
                if license_code not in LICENSE_TYPE_CODES:
                    error = "%s : License code must be a valid license type code.  See https://github.com/HHSIDEAlab/mlvs/blob/master/docs/USProviderLicenseTypesFeb2014.csv" % (
                        d.get('code'))
                    errors.append(error)

    return errors



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars - This software is public domain

from subprocess import call


def load_nppes_and_pecos():
    call(["loadnppes.py", "y", "y", "y"])
    call(["load_pecos.py", "y", "y", "y"])


if __name__ == "__main__":
    load_nppes_and_pecos()



import unittest
import os
import csv
from pdt import chop_nppes_public


# This test suite is meant to be run from the **project root**

class TestChopNppesPublic(unittest.TestCase):

    def test_taxonomy_creation(self):

        csvfile = os.path.join(os.path.dirname(__file__),  "fiftythousand.csv")
        chop_nppes_public.chop_taxonomy(csvfile)

        self.assertTrue(os.stat('tests/fiftythousand_taxonomy.csv'))

        csvfile_taxonomy = os.path.join(os.path.dirname(
            __file__),  "fiftythousand_taxonomy.csv")
        chop_nppes_public.flatten_taxonomy(csvfile_taxonomy)

        self.assertTrue(os.stat('tests/fiftythousand_taxonomy_flat.csv'))

    def test_identifiers_creation(self):

        csvfile = os.path.join(os.path.dirname(__file__),  "fiftythousand.csv")
        chop_nppes_public.chop_identifiers(csvfile)

        self.assertTrue(os.stat('tests/fiftythousand_identifiers.csv'))

        csvfile_identifiers = os.path.join(os.path.dirname(
            __file__),  "fiftythousand_identifiers.csv")
        chop_nppes_public.flatten_identifiers(csvfile_identifiers)

        self.assertTrue(os.stat('tests/fiftythousand_identifiers_flat.csv'))

    def test_chop_licenses_creation(self):

        csvfile = os.path.join(os.path.dirname(__file__),  "fiftythousand.csv")
        chop_nppes_public.chop_licenses(csvfile)

        self.assertTrue(os.stat('tests/fiftythousand_licenses.csv'))

        csvfile_licenses = os.path.join(os.path.dirname(
            __file__),  "fiftythousand_licenses.csv")
        chop_nppes_public.flatten_licenses(csvfile_licenses)

        self.assertTrue(os.stat('tests/fiftythousand_licenses_flat.csv'))

    def test_chop_addresses_creation(self):

        csvfile = os.path.join(os.path.dirname(__file__),  "fiftythousand.csv")
        chop_nppes_public.chop_addresses(csvfile)

        self.assertTrue(os.stat('tests/fiftythousand_addresses.csv'))

        csvfile_addresses = os.path.join(os.path.dirname(
            __file__),  "fiftythousand_addresses.csv")
        chop_nppes_public.flatten_addresses(csvfile_addresses)

        self.assertTrue(os.stat('tests/fiftythousand_addresses_flat.csv'))

    def test_chop_basic_creation(self):

        csvfile = os.path.join(os.path.dirname(__file__),  "fiftythousand.csv")
        chop_nppes_public.chop_basic(csvfile)

        self.assertTrue(os.stat('tests/fiftythousand_basic.csv'))

    def test_chop_other_names_creation(self):
        """Run ensure_provider_indexes should return results """

        csvfile = os.path.join(os.path.dirname(__file__),  "fiftythousand.csv")
        chop_nppes_public.chop_other_names(csvfile)

        self.assertTrue(os.stat('tests/fiftythousand_other_names.csv'))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars - This software is public domain

import os
import sys
import string
import json
import csv
import time
import json_schema_check
from collections import OrderedDict
from datetime import datetime
import ndjson

def newfhir_deactive_stub():
    ps = OrderedDict()

    # ProviderJSON stub
    ps["resourceType"] = "Practitioner"
    ps['identifier'] = [
        {
            "use": "official",
            "system": "http://hl7.org/fhir/sid/us-npi",
                      "value": "",
        }
    ]
    ps['active'] = bool(False)
    return ps


def new_fhir_practitioner_stub(npi, prefix, first_name, last_name, suffix):

    text = "%s %s %s %s %s" % (npi, prefix, first_name, last_name, suffix)
    ps = OrderedDict()
    ps["resourceType"] = "Practitioner"
    ps["text"] = {"status": "generated",
                  "div": "<div><p>%s</p></div>" % (text)
                  }
    ps['extension'] = []
    ps['identifier'] = [
        {
            "use": "official",
            "system": "http://hl7.org/fhir/sid/us-npi",
                      "value": str(npi)
        }
    ]
    ps['active'] = bool(True)
    ps['name'] = [
        {
            "family": [
                last_name
            ],
            "given": [
                first_name
            ],
            "suffix": [
                suffix
            ],
            "prefix": [
                prefix
            ]


        }
    ]
    ps["address"] = []
    ps['telecom'] = []
    ps['qualification'] = []

    return ps


def new_fhir_organization_stub(npi, organization_name):

    text = "%s: NPI= %s (Type 2-Organization/Facility/Pharmacy)" % (organization_name, npi,)
    os = OrderedDict()
    os["resourceType"] = "Organization"
    os["text"] = {"status": "generated",
                  "div": "<div><p>%s</p></div>" % (text)
                  }
    os['extension'] = []
    os['identifier'] = [
        {
            "use": "official",
             "coding": [
                { "system": "http://hl7.org/fhir/v2/0203i",
                  "value": str(npi),
                  "display": "National provider identifier"
                },
                ],
             "text": "US National Provider Identifier"
        }
    ]
    os['name'] = organization_name
    os["address"] = []
    os['telecom'] = []
    os['qualification'] = []
    return os


def publiccsv2fhir(csvfile, output_dir, schema_check=False,
                   include_state_list=[], include_npi_list=[]):
    """Return a response_dict with summary of  publiccsv2fhir transaction."""

    process_start_time = time.time()
    pdir = 1

    # make the output dir
    try:
        os.mkdir(output_dir)
    except:
        pass
    try:
        os.mkdir(os.path.join(output_dir, "Practitioner"))
    except:
        pass
    try:
        os.mkdir(os.path.join(output_dir, "Organization"))
    except:
        pass

    try:
        os.mkdir(os.path.join(output_dir, "Deactive"))
    except:
        pass

    # Make variable/paths for nucc_taxonomy and schema checker, open file
    nucc_tax = os.path.join(os.path.dirname(__file__),
                            "nucc_taxonomy_201.csv")
    # Check which version of Python, and open csv accordingly
    # TODO remove Python2 support
    if sys.version_info[0] < 3:
        csvfile_tax = open(nucc_tax, 'rb')
    else:
        csvfile_tax = open(nucc_tax, 'r', newline='', encoding='iso-8859-1')
    
    tax_reader = csv.reader(csvfile_tax, delimiter=',')
    
    # Load taxonomy
    # read csv file as a list of lists
    list_of_rows = []
    with open(os.path.join(os.path.dirname(__file__),"nucc_taxonomy_201.csv"), 'r', newline='', encoding='iso-8859-1') as read_obj:
        # pass the file object to reader() to get the reader object
        csv_reader = csv.reader(read_obj)
        # Pass reader object to list() to get a list of lists
        list_of_rows = list(csv_reader)
    tax_display = OrderedDict()
    for row in list_of_rows:
        tax_display[row[0]] = "%s, %s, %s" % (row[1], row[2], row[3],)
        
        tax_display[row[0]] = tax_display[row[0]].strip().strip(',')

    
    practitioner_path = os.path.join(os.path.dirname(__file__),
                                     "fhir_json_schema", "Practitioner.json")
    organization_path = os.path.join(os.path.dirname(__file__),
                                     "fhir_json_schema", "Organization.json")

    # Start of opening of csv file to convert and test
    response_dict = OrderedDict()
    fh = open(csvfile, 'r')
    csvhandle = csv.reader(fh, delimiter=',')
    rowindex = 0
    po_count = 0
    practitioner_count = 0
    organization_count = 0
    deactive_count = 0
    error_list = []

    # Create the NDJSON writers.
    out_fh1 = open("Organization.ndjson", 'w')
    organization_writer = ndjson.writer(out_fh1)
    
    out_fh2 = open("Practitioner.ndjson", 'w')
    practitioner_writer = ndjson.writer(out_fh2)

    for row in csvhandle:
        if rowindex == 0:

            rowindex += 1
            column_headers = row

            cleaned_headers = []
            for c in column_headers:
                c = c.replace(".", "")
                c = c.replace("(", "")
                c = c.replace(")", "")
                c = c.replace("$", "-")
                c = c.replace(" ", "_")
                cleaned_headers.append(c)
        else:

            # If the records is not redacted (because its inactive)

            zip_record = list(zip(cleaned_headers, row))
            record = dict(list(zip(cleaned_headers, row)))

            # get rid of blanks
            no_blank_zip = []
            for i in zip_record:
                if i[1]:
                    no_blank_zip.append(i)

            # start our object off with a stub.

            if row[1] == "1":
                r = new_fhir_practitioner_stub(row[0], row[8],  row[6], row[5],
                                               row[9])

            elif row[1] == "2":
                r = new_fhir_organization_stub(row[0], row[4])

            else:
                r = newfhir_deactive_stub()

            # Practice Address

            a = OrderedDict()
            r['address'] = []
            a["use"] = "work"
            a["line"] = []
            a["line"].append(row[28].upper())
            if row[29]:
                a["line"].append(row[29].upper())
            a["city"] = row[30].upper()
            a["state"] = row[31].upper()
            a["postalCode"] = row[32].upper()
            a["country"] = row[33].upper()

            r['address'] = [a]

            # Mailing address --------------------
            if row[1] == "1":
                a['use'] = 'home'
                a['line'] = []
                a['line'].append(row[20].upper())
                if row[21]:
                    a['line'].append(row[21].upper())
                a["city"] = row[22].upper()
                a["state"] = row[23].upper()
                a["postalCode"] = row[24].upper()
                a["country"] = row[25].upper()
                r['address'].append(a)

            # Gender
            if row[1] == "1":
                if row[41] == "M":
                    r["gender"] = "male"
                elif row[41] == "F":
                    r["gender"] = "female"
                elif row[41] != "M" or "F":
                    r["gender"] = "other"
                else:
                    r["gender"] = "unknown"

            # Organization Contact
            if row[43]:
                contact_list = list()
                contact = OrderedDict()
                purpose = OrderedDict()
                purpose['text'] = "Admin"
                contact['purpose'] = purpose
                name = OrderedDict()
                name["family"] = [row[42]]
                name["given"] = [row[43]]
                name["suffix"] = [row[312]]
                name["prefix"] = [row[311]]
                contact['name'] = name

                contact['telecom'] = [
                    {
                        "system": "phone",
                        "value": "%s-%s-%s" % (row[46][0:3],
                                               row[46][3:6],
                                               row[46][6:12]),
                        "use": "work"
                    }
                ]
                contact_list.append(contact)
                r['contact'] = contact_list
                
            # Provider Business Practice Location Address Telephone Number
            t = OrderedDict()
            if row[34]:
                t['system'] = "phone"
                t['value'] = "%s-%s-%s" % (row[34][0:3], row[34][3:6],
                                           row[34][6:12])
                t['use'] = "work"
                r['telecom'] = [t]
            
            # Provider Business Practice Location Address Fax Number
            if row[35]:
                t = OrderedDict()
                t['system'] = "fax"
                t['value'] = "%s-%s-%s" % (row[35][0:3], row[35][3:6],
                                           row[35][6:12])
                t['use'] = "work"
                r['telecom'].append(t)

            # Mailing Phone and Fax numbers for Practitioners
            if row[1] == "1":
                if row[26]:
                    t = OrderedDict()
                    t['system'] = "phone"
                    t['value'] = "%s-%s-%s" % (row[26][0:3], row[26][3:6],
                                               row[26][6:12])
                    t['use'] = 'home'
                    r['telecom'].append(t)

                if row[27]:
                    t = OrderedDict()
                    t['system'] = "fax"
                    t['value'] = "%s-%s-%s" % (row[27][0:3], row[27][3:6],
                                               row[27][6:12])
                    t['use'] = 'home'
                    r['telecom'].append(t)


            # Getting taxonomy codes to store in qualifications
            for i in range(50, 107, 4):
                if row[i] == "Y":

                    taxonomy = OrderedDict()
                    taxonomy["code"] = OrderedDict()
                    # Fill in url
                    #
                    coding = OrderedDict()
                    #extension = OrderedDict()

                    coding['system'] = "http://taxonomy.nucc.org/"
                    coding['code'] = str(row[i - 3])
                    coding['display'] = tax_display[coding['code']]
                    
                    
                    for row_tax in tax_reader:
                        print(row_tax)
                        if coding['code'] == row_tax[0]:
                            coding['display'] = row_tax[2]

                    taxonomy['code']['coding'] = [coding]

                    #extension['url'] = "http://www.nucc.org/"
                    #extension['valueCodeableConcept'] = taxonomy

                    r['qualification'].append(taxonomy)

            # identifiers----------------
            # starting at row 107

            identifier_position = 107
            identifier_code_position = 108
            identifier_state_position = 109
            identifier_issuer_position = 110

            for i in range(1, 50):

                if row[identifier_position]:
                    identifier = OrderedDict()
                    identifier['use']="official"
                    
                    if "MEDICAID" in row[identifier_issuer_position]:
                        code = "MCD"
                        display = "Practitioner Medicaid number"
                    
                    elif "DEA" in row[identifier_issuer_position]:
                        code = "DEA"
                        display = "Drug Enforcement Administration registration number"
    
                    else:
                        code = "AN"
                        display = "Account number"
                        
                    identifier['type']=[{"system":"http://hl7.org/fhir/v2/0203",
                                        "code": code,
                                        "display": display
                                        },]
                    identifier['value'] = row[identifier_position]
                    identifier['text'] = row[identifier_code_position]
                    identifier['state'] = row[
                        identifier_state_position].upper()
                    identifier['issuer'] = row[identifier_issuer_position]
                    r['identifier'].append(identifier)

                identifier_position += 4
                identifier_code_position += 4
                identifier_state_position += 4
                identifier_issuer_position += 4
               
                # licenses 
                # starting at row 49
                license_state_position = 49
                license_number_position = 48

                for i in range(1, 15):
                     # License
                    if row[license_number_position]:
                        license = OrderedDict()
                        license["use"] = "official"
                        license["coding"] = [
                            {"system": "http://hl7.org/fhir/v2/0203",
                             "code": "MD",
                             "display": "Medcial license"},                       ]
                        license['value'] = row[license_number_position]
                        license['state'] = row[license_state_position].upper()
                        license['issuer'] = row[identifier_issuer_position].upper()
                        if not license['issuer'] and license['state']:
                            license['issuer'] = license['state']
                        if not license['issuer']:
                            license['issuer'] = "UNSPECIFIED" 

                        license['text'] = "%s issued by %s"  % (license['value'], license['issuer'])

                        r['qualification'].append(license)
                        
                        #skip to the next
                        license_state_position += 4
                        license_number_position += 4    


            fn = "%s.json" % (row[0])

            if not row[1]:
                deactive_count += 1 
            if r['resourceType'] == "Practitioner":
                # Write to Practitioner NDJSON File
                practitioner_writer.writerow(r)
                practitioner_count +=1 
                # Write to Organization NDJSON File
            elif r['resourceType'] == "Organization":
                organization_writer.writerow(r)
                organization_count += 1 



            #fp = os.path.join(subdir, fn)
            #ofile = open(fp, 'w')
            #ofile.writelines(json.dumps(r, indent=4))
            #ofile.close()
            
            if schema_check:
                if row[1] == "1":
                    results = json_schema_check.json_schema_check(
                        practitioner_path, fp)
                    if results['errors'] != []:
                        response_dict['errors'] = results['errors']
                if row[2] == "2":
                    results = json_schema_check.json_schema_check(
                        organization_path, fp)
                    if results['errors'] != []:
                        response_dict['errors'] = results['errors']
            po_count += 1

            if po_count % 1000 == 0:
                pdir += 1
                out = "%s objects created. Total time is %s seconds." % (
                    po_count, (time.time() - process_start_time))
                print((out))

            rowindex += 1


    response_dict['total_rows_processed'] = rowindex - 1
    response_dict['practitioner_count'] = practitioner_count
    response_dict['organization_count'] = organization_count
    response_dict['deactive_count'] = deactive_count
    response_dict['num_csv_rows'] = rowindex - 1

    out_fh1.close()
    out_fh2.close()
    fh.close()
    csvfile_tax.close()
    return response_dict

if __name__ == "__main__":

    if len(sys.argv) != 3:
        print("Usage:")
        print("csv2fhir_public.py [CSVFILE] [OUTPUT_DIRECTORY]")
        sys.exit(1)

    csv_file = sys.argv[1]
    output_dir = sys.argv[2]

    result = publiccsv2fhir(csv_file, output_dir, include_state_list=[
                                        'WV', 'VA', 'OH', 'MD', 'KY'])
    

    # output the JSON transaction summary
    print((json.dumps(result, indent=4)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
# Written by Alan Viars - This software is public domain

from bs4 import BeautifulSoup
try:
    # For Python 3.0 and later
    from urllib.request import urlopen
except ImportError:
    # Fall back to Python 2's urllib2
    from urllib2 import urlopen
import sys
from subprocess import call


def pull_pecos(download_all=True, Base=False,
               Reassignment=False, Address=False):
    try:

        html_page = urlopen("https://data.cms.gov/public-provider-enrollment")
        file_names = ['Base-Provider-Enrollment-File', 'Reassignment-Sub-File',
                      'Address-Sub-File']

        # Identify file links and parse into a list of strings
        soup = BeautifulSoup(html_page, "html.parser")
        file_links = []
        for file_name in file_names:
            for link in soup.findAll('a'):
                if file_name in link.get('href', ""):
                    file_links.append(list(link.get('href', "").split('/')))

        csv_prefix = 'https://data.cms.gov/api/views/'
        # Possible feature to add is allowing people to name the output file.
        # Currently it supplants the downloaded file supplants a file that is
        # already named the default value.

        # Download files according to user input
        if download_all:
            print("Downloading Base, Address and Reassignment CSV files")
            for csv_link in file_links:
                call(["wget", "--output-document=" + csv_link[-2] + ".csv",
                      csv_prefix + csv_link[-1] + '/rows.csv?accessType=DOWNLOAD'])

        if Base:
            print("Downloading Base CSV file")
            call(["wget", "--output-document=pecos_base.csv", csv_prefix +
                  file_links[0][-1] + '/rows.csv?accessType=DOWNLOAD'])

        if Reassignment:
            print("Downloading Reassignment CSV file")
            call(["wget", "--output-document=pecos_reassignment.csv",
                  csv_prefix + file_links[1][-1] + "/rows.csv?accessType=DOWNLOAD"])

        if Address:
            print("Downloading Address CSV file")
            call(["wget", "--output-document=pecos_address.csv", csv_prefix +
                  file_links[2][-1] + "/rows.csv?accessType=DOWNLOAD"])

    except:
        print(sys.exc_info)


if __name__ == '__main__':
    if len(sys.argv) != 5:
        print("Usage:")
        print("pull_pecos.py [DOWNLOAD ALL Y/N] "
              "[DOWNLOAD BASE Y/N] [DOWNLOAD REASSIGNMENT Y/N] "
              " [DOWNLOAD ADDRESS Y/N]")
        print("Example:")
        print("pull_pecos.py Y N N N")
        sys.exit(1)

    if sys.argv[1] in ("Y", "y", "T", True):
        download_all = True
    else:
        download_all = False

    if sys.argv[2] in ("Y", "y", "T", True):
        Base = True
    else:
        Base = False

    if sys.argv[3] in ("Y", "y", "T", True):
        Reassignment = True
    else:
        Reassignment = False

    if sys.argv[4] in ("Y", "y", "T", True):
        Address = True
    else:
        Address = False

    # Get the file from the command line
    pull_pecos(download_all, Base, Reassignment, Address)
